# -*- coding: utf-8 -*-
#日本語UTF-8

import bpy
from Spr import *
from mathutils import Vector,Quaternion
import math

import time
from imp import reload

import spr_op.visualizer as visualizer


def rad(deg):
    return deg / 180.0 * 3.1415926


# クリーチャのルールを実行するための隔離環境
class CreatureExecutionEnv():
    def __init__(self, ph_con, cr_con):
        self.phSdk          = ph_con.phSdk
        self.phScene        = ph_con.phSdk.GetScene(0)
        self.crSdk          = ph_con.crSdk
        self.creature       = cr_con.creature
        self.body           = cr_con.body
        self.group          = cr_con.group

    def exec(self, text):
        exec(text)


# クリーチャコントローラ
class CreatureController():
    def __init__(self, ph_con, group):
        self.ph_con        = ph_con
        self.ready         = False
        self.group         = group
        
        self.creature      = None
        self.body          = None

        self.group         = group
        self.init_rules    = {}
        self.step_rules    = {}
        self.rule_exec_env = CreatureExecutionEnv(self.ph_con, self)


    def create_creature(self):
        self.ready = False

        crSdk = self.ph_con.crSdk
        
        #-- CRCreature --------------
        crDesc = CRCreatureDesc()
        self.creature = crSdk.CreateCreature(CRCreature().GetIfInfoStatic(),crDesc)
        self.creature.SetName("creature"+self.group.name)
        self.creature.AddChildObject(self.ph_con.phScene)
        
        #-- CRBody ------------------
        crbDesc = CRBodyDesc()
        self.body = self.creature.CreateBody(CRBody().GetIfInfoStatic(),crbDesc)
        self.body.SetName("creature"+self.group.name+"body")
        
        #-- CRBone ------------------
        # Group内のそれぞれのオブジェクトに対し
        for object in self.group.objects:
            if object.name in self.ph_con.solid_controllers:

                # 剛体に対応したオブジェクトであればクリーチャの構成ボーンとして登録する．
                so_con = self.ph_con.solid_controllers[object.name]

                descBone = CRBoneDesc()
                descBone.label = so_con.object.spr_CRIKSolid_label

                if not so_con.phSolid is None:
                    crBone = self.body.CreateObject(CRBone().GetIfInfoStatic(), descBone)
                    crBone.SetName("crbone"+so_con.object.name)
                    crBone.AddChildObject(so_con.phSolid)

                    # IKエンドエフェクタになっていればそれも登録する                            
                    if not so_con.ikeff is None:
                        crBone.AddChildObject(so_con.ikeff)

                    # IK付き関節が設定されていればそれも登録する
                    if (object.name in so_con.joint_controllers) and (not so_con.ika is None):
                        jo_con = so_con.joint_controllers[so_con.object.name]
                        crBone.AddChildObject(jo_con.phJoint)
                        crBone.AddChildObject(so_con.ika)

        # Creatureの行動ルールをセットアップ
        self.update_rules()

        # パラメータをセットアップ
        self.update_parameters()

        # 実行環境を用意し初期化スクリプトを実行
        self.rule_exec_env = CreatureExecutionEnv(self.ph_con, self)
        for key,rule in sorted(self.init_rules.items()):
            try:
                self.rule_exec_env.exec(rule)
            except Exception as exception:#例外情報の記録、スクリプト実行の停止
                self.ph_con.info["error"]  ="" #空にする
                self.ph_con.info["error"] +="Error in running creature initialize rules.\n\n"
                self.ph_con.info["error"] += "Script Name: " + key +"\n"
                self.ph_con.info["error"] += "Type: " + str(type(exception)) +"\n"
                self.ph_con.info["error"] += "Args: " + str(exception.args) +"\n"
                #self.ph_con.info["error"] += "Message: " + exception.message
                self.ph_con.info["error"] += "Exception: " + str(exception) +"\n"
                
                print("Error in running init rules")
                print("Error message is in Text \"spr_error\"")
                print(str(exception))
                
                bpy.context.scene.spr_run_scripts_enabled = 0
                print("Stopped Running scripts")
                
                return()

        # 準備完了
        self.ready = True
            

    def update_rules(self):
        self.init_rules.clear()
        self.step_rules.clear()

        # -- init --
        rule_name = "creature_rule_"+self.group.name+"_init"
        try:#ルールが既存ならそちらを使う
            rule_text = bpy.data.texts[rule_name]
        except:#無ければデフォルトの物を作る
            rule_text = bpy.data.texts.new(rule_name)
            rule  = "#" + rule_name + "\n"
            rule += "#" + "This is an initialize rule for the creature "+self.group.name + ". \n"
            rule_text.from_string(rule)
        self.init_rules[rule_name] = rule_text.as_string()

        # -- step --
        rule_name = "creature_rule_"+self.group.name+"_step"
        try:#ルールが既存ならそちらを使う
            rule_text = bpy.data.texts[rule_name]
        except:#無ければデフォルトの物を作る
            rule_text = bpy.data.texts.new(rule_name)
            rule  = "#" + rule_name + "\n"
            rule += "#" + "This is a step rule for the creature "+self.group.name + ". \n"
            rule_text.from_string(rule)
        self.step_rules[rule_name] = rule_text.as_string()
        

    def update_parameters(self):
        #spring/damper        
        spring_ratio = self.group.spr_creature_spring_ratio
        damper_ratio = self.group.spr_creature_damper_ratio
        for crBone in self.body.GetBones():
            crBone_name = crBone.GetName()
            obj_name    = crBone_name.lstrip("crbone")
            obj         = self.ph_con.solid_controllers[obj_name].object
            if crBone.GetPHJoint():
                crBone.GetPHJoint().SetSpring( obj.spr_spring * spring_ratio )
                crBone.GetPHJoint().SetDamper( obj.spr_damper * damper_ratio )

    
    def step(self):
        for key,rule in sorted(self.step_rules.items()):
            try:
                self.rule_exec_env.exec(rule)
            except Exception as exception:#例外情報の記録、スクリプト実行の停止
                self.ph_con.info["error"]  ="" #空にする
                self.ph_con.info["error"] +="Error in running creature step rules.\n\n"
                self.ph_con.info["error"] += "Script Name: " + key +"\n"
                self.ph_con.info["error"] += "Type: " + str(type(exception)) +"\n"
                self.ph_con.info["error"] += "Args: " + str(exception.args) +"\n"
                #self.ph_con.info["error"] += "Message: " + exception.message
                self.ph_con.info["error"] += "Exception: " + str(exception) +"\n"
                
                print("Error in running step rules")
                print("Error message is in Text \"spr_error\"")
                print(str(exception))
                
                bpy.context.scene.spr_run_scripts_enabled = 0
                print("Stopped Running scripts")
                
                return()


