# -*- coding: utf-8 -*-
import Spr
from   Spr           import *
from   mathutils     import *
import bpy
from   spr_op.util   import *

class SpaceNavigator:
	# パラメータ
	## -- 最大速度 [m/s]
	maxVelocity = 30.0

	## -- 最大角速度 [rad/s]
	maxAngularVelocity = rad(300)

	## -- 視野にあわせて座標変換するか
	bCameraCoord = True

	## -- 視野内に制限するか
	bLimitInSight = True

	## -- 奥行き方向の移動制限の範囲
	depthFar  = 50
	depthNear =  3

	# ----- ----- ----- ----- ----- ----- ----- ----- ----- -----
	def __init__(self, spcNav):
		# 初期設定
		self.spcNav = spcNav
		self.spcNav.SetMaxVelocity(self.maxVelocity)
		self.spcNav.SetMaxAngularVelocity(self.maxAngularVelocity)
		self.soCtlTarget = None

		if hasattr(Spr,"app"):
			app.UseSpaceNavigator(True)

	def set_control_target(self, soCtlTarget):
		self.soCtlTarget = soCtlTarget

	def step(self):
		if self.soCtlTarget==None:
			return 
		phScene = self.soCtlTarget.GetScene()

		# 現在のView/Perspective行列を取ってくる
		## -- VIEW_3D areaを見つけてくる
		region_3d = None
		for area in bpy.context.screen.areas:
			if area.type=="VIEW_3D":
				region_3d = area.spaces[0].region_3d

		if region_3d==None:
			return

		## -- 行列を取得する
		mView    = region_3d.view_matrix
		mPers    = region_3d.perspective_matrix
		mViewInv = mView.inverted()
		mPersInv = mPers.inverted()

		# SpaceNavigatorの移動量を計算する
		dV = self.spcNav.GetVelocity()        * phScene.GetTimeStep()
		dQ = self.spcNav.GetAngularVelocity() * phScene.GetTimeStep()

		# ターゲットの新しい座標を計算する
		if self.bCameraCoord:
			## -- 視野に合わせて座標変換する
			q    = mView.to_3x3().to_quaternion()
			ori0 = Quaterniond(q[0], q[1], q[2], q[3]).Inv()
			pos0 = self.soCtlTarget.GetPose().getPos()
			pos1 = ori0*dV + pos0

			ori0_= self.soCtlTarget.GetPose().getOri()
			ori1 = Quaterniond.Rot(ori0*dQ) * Quaterniond(ori0_.w, ori0_.x, ori0_.y, ori0_.z)
		else:
			pos0 = self.soCtlTarget.GetPose().getPos()
			pos1 = dV + pos0

			ori0_= self.soCtlTarget.GetPose().getOri()
			ori1 = Quaterniond.Rot(dQ) * Quaterniond(ori0_.w, ori0_.x, ori0_.y, ori0_.z)

		# ターゲットの位置を視野内に制限する
		pos_screen  = mPers * Vector((pos1.x, pos1.y, pos1.z, 1))
		w           = pos_screen[3]
		pos_screen *= (1.0 / w)

		pos_limited      = pos_screen
		if self.bLimitInSight:
			pos_limited.x    = min(max( -1.0,  pos_limited.x),  1.0)
			pos_limited.y    = min(max( -1.0,  pos_limited.y),  1.0)
		pos_limited_3d   = mPersInv * ( Vector((pos_limited.x, pos_limited.y, pos_limited.z, 1))*w )

		pos_limited_3d   = mView * pos_limited_3d
		if self.bLimitInSight:
			pos_limited_3d.z = min(max( -self.depthFar,  pos_limited_3d.z),  -self.depthNear)
		pos_limited_3d   = mViewInv * pos_limited_3d

		# ターゲットの位置・姿勢に反映
		self.soCtlTarget.SetFramePosition(Vec3d(pos_limited_3d.x, pos_limited_3d.y, pos_limited_3d.z))
		self.soCtlTarget.SetOrientation(ori1)

