# -*- coding: utf-8 -*-
#日本語UTF-8

import bpy
import Spr
import math
from math import sin, cos, tan, pi, radians

import spr_op.bl_util as bl_util

def rad(deg):
    return deg / 180.0 * 3.1415926


class ConeVisualizer():
	def __init__(self, name, kind):
		self.name   = name
		self.kind   = kind
		self.object = None

		self.alt    = rad(45)
		self.az     = rad(45)
		
		self.last_az  = self.az
		self.last_alt = self.alt

		self.prepare_object()

	def attach_to(self, object):
		self.object.parent = object

	def update(self, alt, az):
		self.alt = alt
		self.az  = az
		
		x = 1; y = 1; bChanged = False

		if (not self.az==self.last_az) or (not self.alt==self.last_alt):
			x = tan(rad(self.az))
			y = tan(rad(self.alt))
			self.object.scale = (x, y, 1)

			self.last_az  = self.az
			self.last_alt = self.alt

	# ----- ----- ----- ----- ----- ----- ----- ----- ----- -----

	def show(self):
		pass

	def hide(self):
		pass

	def prepare_object(self):
		obj_name = "ConeVisualizerObj." + self.name
		mat_name = "ConeVisualizerMat." + self.kind
		try:
			self.object = bpy.data.objects[obj_name]
		except:
			# -- Create New Cone --
			depth        = 10.0
			vertices_num = 32
			radius_alt   = 10.0
			radius_az    = 10.0

			#2D array {[0,0,0],[0,0,0],...}
			vertices = [[0 for i in range(0, 3, 1)] for j in range(0, vertices_num + 2, 1)]

			vertices[0] = (0,0,0)
			i = 1
			while i <= vertices_num:
				vertices[i][0] =  radius_az  * cos(i * pi/(vertices_num/2))
				vertices[i][1] =  radius_alt * sin(i * pi/(vertices_num/2))
				vertices[i][2] = -depth
				i += 1
				
			vertices[vertices_num + 1] = (0, 0, -depth)
				
			i = 0
			faces = [[0 for i in range(0, 3, 1)] for j in range(0, vertices_num*2, 1)]
			while i < (vertices_num - 1):
				faces[i]                = (0,   i+1, i+2)
				faces[vertices_num + i] = (i+2, i+1, vertices_num+1)
				i += 1
				
			faces[vertices_num   - 1] = (0, vertices_num, 1)
			faces[vertices_num*2 - 1] = (1, vertices_num, vertices_num+1)
				
			self.object = bl_util.create_mesh(obj_name, (0,0,0), vertices, [], faces)
				
			self.object.rotation_euler = (radians(90.0), 0.0, 0.0)
				
			#描画関連
			bpy.ops.object.select_all(action='DESELECT')
			self.object.select = True
			bpy.ops.object.shade_smooth()
			
			self.object.show_x_ray  = True #Trueにするとbuffer shadowがでなくなる・・・
			self.object.hide_select = True
				
			#Set Material
			mat = bl_util.prepare_material(mat_name, (0,1,0), (0.5,0.5,0), 0.2)
			bl_util.set_material(self.object, mat)
			



